@@ -50,9 +50,8 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         override = \
             role == "text frame" \
             or (role == pyatspi.ROLE_PARAGRAPH \
-                and self._script.getAncestor(obj,
-                                             [pyatspi.ROLE_DIALOG],
-                                             [pyatspi.ROLE_APPLICATION]))
+                and self._script.utilities.ancestorWithRole(
+                      obj, [pyatspi.ROLE_DIALOG], [pyatspi.ROLE_APPLICATION]))
         return override
 
     def _generateRoleName(self, obj, **args):
@@ -93,20 +92,46 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
     def _generateLabel(self, obj, **args):
         """Returns the label for an object as an array of strings (and
         possibly voice and audio specifications).  The label is
-        determined by the getDisplayedLabel of the script, and an
-        empty array will be returned if no label can be found.
+        determined by the displayedLabel method of the script utility,
+        and an empty array will be returned if no label can be found.
         """
         result = []
+        acss = self.voice(speech_generator.DEFAULT)
         override = self.__overrideParagraph(obj, **args)
-        label = self._script.getDisplayedLabel(obj) or ""
+        label = self._script.utilities.displayedLabel(obj) or ""
         if not label and override:
-            label = self._script.getDisplayedLabel(obj.parent) or ""
+            label = self._script.utilities.displayedLabel(obj.parent) or ""
         result.append(label.strip())
+        if result:
+            result.extend(acss)
         return result
 
+    def _generateName(self, obj, **args):
+        """Returns an array of strings for use by speech and braille that
+        represent the name of the object.  If the object is directly
+        displaying any text, that text will be treated as the name.
+        Otherwise, the accessible name of the object will be used.  If
+        there is no accessible name, then the description of the
+        object will be used.  This method will return an empty array
+        if nothing can be found.
+        """
+        role = args.get('role', obj.getRole())
+        if role in [pyatspi.ROLE_PUSH_BUTTON, pyatspi.ROLE_TOGGLE_BUTTON] \
+           and self._script.utilities.ancestorWithRole(
+                obj, [pyatspi.ROLE_TOOL_BAR], [pyatspi.ROLE_FRAME]):
+            acss = self.voice(speech_generator.SYSTEM)
+            result = [obj.name]
+            if result:
+                result.extend(acss)
+                return result
+
+        return speech_generator.SpeechGenerator._generateName(
+            self, obj, **args)
+
     def _generateLabelOrName(self, obj, **args):
         """Gets the label or the name if the label is not preset."""
         result = []
+        acss = self.voice(speech_generator.DEFAULT)
         override = self.__overrideParagraph(obj, **args)
         # Treat a paragraph which is serving as a text entry in a dialog
         # as a text object.
@@ -126,11 +151,27 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                 #
                 if not parentLabel and obj.name and len(obj.name):
                     result.append(obj.name)
+                if result:
+                    result.extend(acss)
         else:
             result.extend(speech_generator.SpeechGenerator._generateLabelOrName(
                 self, obj, **args))
         return result
 
+    def _generateAvailability(self, obj, **args):
+        """Returns an array of strings for use by speech and braille that
+        represent the grayed/sensitivity/availability state of the
+        object, but only if it is insensitive (i.e., grayed out and
+        inactive).  Otherwise, and empty array will be returned.
+        """
+
+        result = []
+        if not self._script.isSpreadSheetCell(obj, startFromTable=True):
+            result.extend(speech_generator.SpeechGenerator.\
+                _generateAvailability(self, obj, **args))
+
+        return result
+
     def _generateDescription(self, obj, **args):
         """Returns an array of strings (and possibly voice and audio
         specifications) that represent the description of the object,
@@ -138,12 +179,13 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         label.
         """
         result = []
+        acss = self.voice(speech_generator.SYSTEM)
         if obj.description:
             # The description of some OOo paragraphs consists of the name
             # and the displayed text, with punctuation added. Try to spot
             # this and, if found, ignore the description.
             #
-            text = self._script.getDisplayedText(obj) or ""
+            text = self._script.utilities.displayedText(obj) or ""
             desc = obj.description.replace(text, "")
             for item in obj.name.split():
                 desc = desc.replace(item, "")
@@ -152,12 +194,15 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                     result.append(obj.description)
                     break
 
+        if result:
+            result.extend(acss)
         return result
 
     def _generateToggleState(self, obj, **args):
         """Treat push buttons (which act like toggle buttons) and toggle
         buttons in the toolbar specially.  This is so we can have more
         natural sounding speech such as "bold on", "bold off", etc."""
+        acss = self.voice(speech_generator.SYSTEM)
         result = []
         role = args.get('role', obj.getRole())
         if role in [pyatspi.ROLE_PUSH_BUTTON, pyatspi.ROLE_TOGGLE_BUTTON] \
@@ -170,6 +215,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                 # Translators: this represents the state of a check box
                 #
                 result.append(_("off"))
+            result.extend(acss)
         elif role == pyatspi.ROLE_TOGGLE_BUTTON:
             result.extend(speech_generator.SpeechGenerator._generateToggleState(
                 self, obj, **args))
@@ -183,12 +229,13 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         row header(s).
         """
         result = []
+        acss = self.voice(speech_generator.DEFAULT)
         try:
             table = obj.parent.queryTable()
         except:
             pass
         else:
-            index = self._script.getCellIndex(obj)
+            index = self._script.utilities.cellIndex(obj)
             rowIndex = table.getRowAtIndex(index)
             if rowIndex >= 0 \
                and table in self._script.dynamicRowHeaders:
@@ -200,17 +247,20 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                     headerText = None
                 if header.childCount > 0:
                     for child in header:
-                        text = self._script.getText(child, 0, -1)
+                        text = self._script.utilities.substring(child, 0, -1)
                         if text:
                             result.append(text)
                 elif headerText:
-                    text = self._script.getText(header, 0, -1)
+                    text = self._script.utilities.substring(header, 0, -1)
                     if text:
                         result.append(text)
+        if result:
+            result.extend(acss)
         return result
 
     def _generateNewRowHeader(self, obj, **args):
         result = []
+        acss = self.voice(speech_generator.DEFAULT)
         # Check to see if this spread sheet cell has either a dynamic
         # row heading associated with it.
         #
@@ -220,7 +270,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
             parentTable = parent.queryTable()
         except NotImplementedError:
             parentTable = None
-        index = self._script.getCellIndex(obj)
+        index = self._script.utilities.cellIndex(obj)
         if "lastRow" in self._script.pointOfReference and \
            self._script.pointOfReference["lastRow"] != \
            parentTable.getRowAtIndex(index):
@@ -233,13 +283,15 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                     headerText = None
                 if header.childCount > 0:
                     for child in header:
-                        text = self._script.getText(child, 0, -1)
+                        text = self._script.utilities.substring(child, 0, -1)
                         if text:
                             result.append(text)
                 elif headerText:
-                    text = self._script.getText(header, 0, -1)
+                    text = self._script.utilities.substring(header, 0, -1)
                     if text:
                         result.append(text)
+        if result:
+            result.extend(acss)
         return result
 
     def _generateColumnHeader(self, obj, **args):
@@ -250,12 +302,13 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         column header(s).
         """
         result = []
+        acss = self.voice(speech_generator.DEFAULT)
         try:
             table = obj.parent.queryTable()
         except:
             pass
         else:
-            index = self._script.getCellIndex(obj)
+            index = self._script.utilities.cellIndex(obj)
             columnIndex = table.getColumnAtIndex(index)
             if columnIndex >= 0 \
                and table in self._script.dynamicColumnHeaders:
@@ -267,17 +320,20 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                     headerText = None
                 if header.childCount > 0:
                     for child in header:
-                        text = self._script.getText(child, 0, -1)
+                        text = self._script.utilities.substring(child, 0, -1)
                         if text:
                             result.append(text)
                 elif headerText:
-                    text = self._script.getText(header, 0, -1)
+                    text = self._script.utilities.substring(header, 0, -1)
                     if text:
                         result.append(text)
+        if result:
+            result.extend(acss)
         return result
 
     def _generateNewColumnHeader(self, obj, **args):
         result = []
+        acss = self.voice(speech_generator.DEFAULT)
         # Check to see if this spread sheet cell has either a dynamic
         # row heading associated with it.
         #
@@ -287,7 +343,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
             parentTable = parent.queryTable()
         except NotImplementedError:
             parentTable = None
-        index = self._script.getCellIndex(obj)
+        index = self._script.utilities.cellIndex(obj)
         if "lastColumn" in self._script.pointOfReference and \
            self._script.pointOfReference["lastColumn"] != \
            parentTable.getColumnAtIndex(index):
@@ -300,13 +356,15 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                     headerText = None
                 if header.childCount > 0:
                     for child in header:
-                        text = self._script.getText(child, 0, -1)
+                        text = self._script.utilities.substring(child, 0, -1)
                         if text:
                             result.append(text)
                 elif headerText:
-                    text = self._script.getText(header, 0, -1)
+                    text = self._script.utilities.substring(header, 0, -1)
                     if text:
                         result.append(text)
+        if result:
+            result.extend(acss)
         return result
 
     def _generateTooLong(self, obj, **args):
@@ -318,9 +376,11 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         of the spread sheet cell, or None if the message fits.
         """
         result = []
+        acss = self.voice(speech_generator.SYSTEM)
         try:
             text = obj.queryText()
-            objectText = self._script.getText(obj, 0, -1).decode("UTF-8")
+            objectText = \
+                self._script.utilities.substring(obj, 0, -1).decode("UTF-8")
             extents = obj.queryComponent().getExtents(pyatspi.DESKTOP_COORDS)
         except NotImplementedError:
             pass
@@ -341,16 +401,19 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                 result = [ngettext("%d character too long",
                                    "%d characters too long",
                                    tooLongCount) % tooLongCount]
+        if result:
+            result.extend(acss)
         return result
 
     def _generateSpreadSheetCell(self, obj, **args):
         result = []
+        acss = self.voice(speech_generator.DEFAULT)
         if self._script.inputLineForCell == None:
             self._script.inputLineForCell = \
                 self._script.locateInputLine(obj)
         try:
             if obj.queryText():
-                objectText = self._script.getText(obj, 0, -1)
+                objectText = self._script.utilities.substring(obj, 0, -1)
                 if (not script_settings.speakSpreadsheetCoordinates \
                     or args.get('formatType', 'unfocused') == 'basicWhereAmI') \
                    and len(objectText) == 0:
@@ -359,6 +422,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                     #
                     objectText = _("blank")
                 result.append(objectText)
+                result.extend(acss)
         except NotImplementedError:
             pass
 
@@ -377,6 +441,9 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                         result.append(name)
                         break
 
+        if result:
+            result.extend(acss)
+
         tooLong = self._generateTooLong(obj, **args)
         if tooLong and len(tooLong):
             result.extend(self._generatePause(obj, **args))
@@ -429,7 +496,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
             if settings.readTableCellRow:
                 parent = obj.parent
                 parentTable = parent.queryTable()
-                index = self._script.getCellIndex(obj)
+                index = self._script.utilities.cellIndex(obj)
                 row = parentTable.getRowAtIndex(index)
                 column = parentTable.getColumnAtIndex(index)
                 # This is an indication of whether we should speak all the
@@ -469,6 +536,21 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                 result.append(_("blank"))
         return result
 
+    def _generateEndOfTableIndicator(self, obj, **args):
+        """Returns an array of strings (and possibly voice and audio
+        specifications) indicating that this cell is the last cell
+        in the table. Overridden here because Orca keeps saying "end
+        of table" in certain lists (e.g. the Templates and Documents
+        dialog).
+        """
+
+        topLevel = self._script.utilities.topLevelObject(obj)
+        if topLevel and topLevel.getRole() == pyatspi.ROLE_DIALOG:
+            return []
+
+        return (speech_generator.SpeechGenerator._generateEndOfTableIndicator(
+                self, obj, **args))
+
     def generateSpeech(self, obj, **args):
         result = []
         if args.get('formatType', 'unfocused') == 'basicWhereAmI' \
