@@ -225,6 +225,10 @@ quitModule = "orca_quit"
 #
 findModule = "orca_gui_find"
 
+# The name of the module that holds the splash screen.
+#
+splashModule = "orca_gui_splash"
+
 # A list of keys that can serve as the Orca modifier key.  The list is
 # so we can provide better cross platform support (e.g., Sun keyboard
 # vs. PC-104 keyboard layouts).  When any of these keys is pressed,
@@ -373,11 +377,13 @@ speechServerInfo        = None # None means let the factory decide.
 DEFAULT_VOICE           = "default"
 UPPERCASE_VOICE         = "uppercase"
 HYPERLINK_VOICE         = "hyperlink"
+SYSTEM_VOICE            = "system"
 
 voices = {
     DEFAULT_VOICE   : ACSS({}),
     UPPERCASE_VOICE : ACSS({ACSS.AVERAGE_PITCH : 5.6}),
-    HYPERLINK_VOICE : ACSS({})
+    HYPERLINK_VOICE : ACSS({}),
+    SYSTEM_VOICE    : ACSS({}),
 }
 
 # If True, enable speaking of speech indentation and justification.
@@ -1535,16 +1541,16 @@ DATE_FORMAT_NUMBERS_MD = "%m/%d"
 DATE_FORMAT_NUMBERS_DMY = "%d/%m/%Y"
 DATE_FORMAT_NUMBERS_MDY = "%m/%d/%Y"
 DATE_FORMAT_NUMBERS_YMD = "%Y/%m/%d"
-DATE_FORMAT_FULL_DM = "%A, %d %B"
-DATE_FORMAT_FULL_MD = "%A, %B %d"
-DATE_FORMAT_FULL_DMY = "%A, %d %B, %Y"
-DATE_FORMAT_FULL_MDY = "%A, %B %d, %Y"
-DATE_FORMAT_FULL_YMD = "%Y. %B %d, %A."
-DATE_FORMAT_ABBREVIATED_DM = "%a, %d %b"
-DATE_FORMAT_ABBREVIATED_MD = "%a, %b %d"
-DATE_FORMAT_ABBREVIATED_DMY = "%a, %d %b, %Y"
-DATE_FORMAT_ABBREVIATED_MDY = "%a, %b %d, %Y"
-DATE_FORMAT_ABBREVIATED_YMD = "%Y. %b %d, %a."
+DATE_FORMAT_FULL_DM = "%A, %-d %B"
+DATE_FORMAT_FULL_MD = "%A, %B %-d"
+DATE_FORMAT_FULL_DMY = "%A, %-d %B, %Y"
+DATE_FORMAT_FULL_MDY = "%A, %B %-d, %Y"
+DATE_FORMAT_FULL_YMD = "%Y. %B %-d, %A."
+DATE_FORMAT_ABBREVIATED_DM = "%a, %-d %b"
+DATE_FORMAT_ABBREVIATED_MD = "%a, %b %-d"
+DATE_FORMAT_ABBREVIATED_DMY = "%a, %-d %b, %Y"
+DATE_FORMAT_ABBREVIATED_MDY = "%a, %b %-d, %Y"
+DATE_FORMAT_ABBREVIATED_YMD = "%Y. %b %-d, %a."
 
 # To keep Orca from spitting up upon launch.
 #
