@@ -63,6 +63,22 @@ LINE_BREAK = [LineBreak()]
 #
 METHOD_PREFIX = "_generate"
 
+DEFAULT        = "default"
+UPPERCASE      = "uppercase"
+HYPERLINK      = "hyperlink"
+SYSTEM         = "system"
+STATE          = "state" # Candidate for sound
+VALUE          = "value" # Candidate for sound
+
+voiceType = {
+    DEFAULT     : settings.DEFAULT_VOICE,
+    UPPERCASE   : settings.UPPERCASE_VOICE,
+    HYPERLINK   : settings.HYPERLINK_VOICE,
+    SYSTEM      : settings.SYSTEM_VOICE,
+    STATE       : settings.SYSTEM_VOICE, # Users may prefer DEFAULT_VOICE here
+    VALUE       : settings.SYSTEM_VOICE, # Users may prefer DEFAULT_VOICE here
+}
+
 class SpeechGenerator(generator.Generator):
     """Takes accessible objects and produces a string to speak for
     those objects.  See the generateSpeech method, which is the primary
@@ -105,6 +121,74 @@ class SpeechGenerator(generator.Generator):
     #                                                                   #
     #####################################################################
 
+    def _generateName(self, obj, **args):
+        """Returns an array of strings for use by speech and braille that
+        represent the name of the object.  If the object is directly
+        displaying any text, that text will be treated as the name.
+        Otherwise, the accessible name of the object will be used.  If
+        there is no accessible name, then the description of the
+        object will be used.  This method will return an empty array
+        if nothing can be found.  [[[WDW - I wonder if we should just
+        have _generateName, _generateDescription,
+        _generateDisplayedText, etc., that don't do any fallback.
+        Then, we can allow the formatting to do the fallback (e.g.,
+        'displayedText or name or description'). [[[JD to WDW - I
+        needed a _generateDescription for whereAmI. :-) See below.
+        """
+        acss = self.voice(DEFAULT)
+        result = generator.Generator._generateName(self, obj, **args)
+        if result:
+            result.extend(acss)
+        return result
+
+    def _generateLabel(self, obj, **args):
+        """Returns the label for an object as an array of strings for use by
+        speech and braille.  The label is determined by the displayedLabel
+        method of the script utility, and an empty array will be returned if
+        no label can be found.
+        """
+        acss = self.voice(DEFAULT)
+        result = generator.Generator._generateLabel(self, obj, **args)
+        if result:
+            result.extend(acss)
+        return result
+
+    def _generateLabelOrName(self, obj, **args):
+        """Returns the label as an array of strings for speech and braille.
+        If the label cannot be found, the name will be used instead.
+        If the name cannot be found, an empty array will be returned.
+        """
+        result = []
+        acss = self.voice(DEFAULT)
+        result.extend(self._generateLabel(obj, **args))
+        if not result:
+            if obj.name and (len(obj.name)):
+                result.append(obj.name)
+                result.extend(acss)
+        return result
+
+    def _generateDescription(self, obj, **args):
+        """Returns an array of strings fo use by speech and braille that
+        represent the description of the object, if that description
+        is different from that of the name and label.
+        """
+        acss = self.voice(SYSTEM)
+        result = generator.Generator._generateDescription(self, obj, **args)
+        if result:
+            result.extend(acss)
+        return result
+
+    def _generateReadOnly(self, obj, **args):
+        """Returns an array of strings for use by speech and braille that
+        represent the read only state of this object, but only if it
+        is read only (i.e., it is a text area that cannot be edited).
+        """
+        acss = self.voice(SYSTEM)
+        result = generator.Generator._generateReadOnly(self, obj, **args)
+        if result:
+            result.extend(acss)
+        return result
+
     def _generateTextRole(self, obj, **args):
         """A convenience method to prevent the pyatspi.ROLE_PARAGRAPH role
         from being spoken. In the case of a pyatspi.ROLE_PARAGRAPH
@@ -131,9 +215,11 @@ class SpeechGenerator(generator.Generator):
         accessible role of the obj.
         """
         result = []
+        acss = self.voice(SYSTEM)
         role = args.get('role', obj.getRole())
         if (role != pyatspi.ROLE_UNKNOWN):
             result.append(rolenames.getSpeechForRoleName(obj, role))
+            result.extend(acss)
         return result
 
     def getRoleName(self, obj, **args):
@@ -152,11 +238,14 @@ class SpeechGenerator(generator.Generator):
         hierarchy and which are not in a label for or labelled by
         relation.
         """
-        labels = self._script.utilities.unrelatedLabels(obj)
         result = []
+        acss = self.voice(DEFAULT)
+        labels = self._script.utilities.unrelatedLabels(obj)
         for label in labels:
             name = self._generateName(label, **args)
             result.extend(name)
+        if result:
+            result.extend(acss)
         return result
 
     def _generateEmbedded(self, obj, **args):
@@ -165,12 +254,15 @@ class SpeechGenerator(generator.Generator):
         This either is the label or name of the object or the name of
         the application for the object.
         """
+        acss = self.voice(DEFAULT)
         result = self._generateLabelOrName(obj, **args)
         if not result:
             try:
                 result.append(obj.getApplication().name)
             except:
                 pass
+        if result:
+            result.extend(acss)
         return result
 
     #####################################################################
@@ -179,6 +271,42 @@ class SpeechGenerator(generator.Generator):
     #                                                                   #
     #####################################################################
 
+    def _generateCheckedState(self, obj, **args):
+        """Returns an array of strings for use by speech and braille that
+        represent the checked state of the object.  This is typically
+        for check boxes. [[[WDW - should we return an empty array if
+        we can guarantee we know this thing is not checkable?]]]
+        """
+        acss = self.voice(STATE)
+        result = generator.Generator._generateCheckedState(self, obj, **args)
+        if result:
+            result.extend(acss)
+        return result
+
+    def _generateExpandableState(self, obj, **args):
+        """Returns an array of strings for use by speech and braille that
+        represent the expanded/collapsed state of an object, such as a
+        tree node. If the object is not expandable, an empty array
+        will be returned.
+        """
+        acss = self.voice(STATE)
+        result = generator.Generator._generateExpandableState(self, obj, **args)
+        if result:
+            result.extend(acss)
+        return result
+
+    def _generateMenuItemCheckedState(self, obj, **args):
+        """Returns an array of strings for use by speech and braille that
+        represent the checked state of the menu item, only if it is
+        checked. Otherwise, and empty array will be returned.
+        """
+        acss = self.voice(STATE)
+        result = generator.Generator.\
+            _generateMenuItemCheckedState(self, obj, **args)
+        if result:
+            result.extend(acss)
+        return result
+
     def _generateMultiselectableState(self, obj, **args):
         """Returns an array of strings (and possibly voice and audio
         specifications) that represent the multiselectable state of
@@ -186,6 +314,7 @@ class SpeechGenerator(generator.Generator):
         is not multiselectable, an empty array will be returned.
         """
         result = []
+        acss = self.voice(STATE)
         if obj.getState().contains(pyatspi.STATE_MULTISELECTABLE):
             # Translators: "multi-select" refers to a web form list
             # in which more than one item can be selected at a time.
@@ -193,6 +322,31 @@ class SpeechGenerator(generator.Generator):
             result.append(self._script.formatting.getString(
                 mode='speech',
                 stringType='multiselect'))
+            result.extend(acss)
+        return result
+
+    def _generateRadioState(self, obj, **args):
+        """Returns an array of strings for use by speech and braille that
+        represent the checked state of the object.  This is typically
+        for check boxes. [[[WDW - should we return an empty array if
+        we can guarantee we know this thing is not checkable?]]]
+        """
+        acss = self.voice(STATE)
+        result = generator.Generator._generateRadioState(self, obj, **args)
+        if result:
+            result.extend(acss)
+        return result
+
+    def _generateToggleState(self, obj, **args):
+        """Returns an array of strings for use by speech and braille that
+        represent the checked state of the object.  This is typically
+        for check boxes. [[[WDW - should we return an empty array if
+        we can guarantee we know this thing is not checkable?]]]
+        """
+        acss = self.voice(STATE)
+        result = generator.Generator._generateToggleState(self, obj, **args)
+        if result:
+            result.extend(acss)
         return result
 
     #####################################################################
@@ -207,6 +361,7 @@ class SpeechGenerator(generator.Generator):
         the link associated with obj.
         """
         result = []
+        acss = self.voice(HYPERLINK)
         # Get the URI for the link of interest and parse it. The parsed
         # URI is returned as a tuple containing six components:
         # scheme://netloc/path;parameters?query#fragment.
@@ -250,6 +405,8 @@ class SpeechGenerator(generator.Generator):
                 result.append(linkOutput)
                 if obj.childCount and obj[0].getRole() == pyatspi.ROLE_IMAGE:
                     result.extend(self._generateRoleName(obj[0]))
+        if result:
+            result.extend(acss)
         return result
 
     def _generateSiteDescription(self, obj, **args):
@@ -258,6 +415,7 @@ class SpeechGenerator(generator.Generator):
         pointed to by the URI of the link associated with obj.
         """
         result = []
+        acss = self.voice(HYPERLINK)
         link_uri = self._script.utilities.uri(obj)
         if link_uri:
             link_uri_info = urlparse.urlparse(link_uri)
@@ -297,6 +455,8 @@ class SpeechGenerator(generator.Generator):
                     # site than that of the link.
                     #
                     result.append(_("different site"))
+        if result:
+            result.extend(acss)
         return result
 
     def _generateFileSize(self, obj, **args):
@@ -306,6 +466,7 @@ class SpeechGenerator(generator.Generator):
         obj.
         """
         result = []
+        acss = self.voice(HYPERLINK)
         sizeString = ""
         uri = self._script.utilities.uri(obj)
         if not uri:
@@ -332,6 +493,8 @@ class SpeechGenerator(generator.Generator):
                 # Translators: This is the size of a file in megabytes
                 #
                 result.append(_("%.2f megabytes") % (float(size) * .000001))
+        if result:
+            result.extend(acss)
         return result
 
     #####################################################################
@@ -346,6 +509,7 @@ class SpeechGenerator(generator.Generator):
         it exists.  Otherwise, an empty array is returned.
         """
         result = []
+        acss = self.voice(DEFAULT)
         try:
             image = obj.queryImage()
         except:
@@ -353,6 +517,7 @@ class SpeechGenerator(generator.Generator):
         else:
             args['role'] = pyatspi.ROLE_IMAGE
             result.extend(self.generate(obj, **args))
+            result.extend(acss)
         return result
 
     #####################################################################
@@ -372,6 +537,7 @@ class SpeechGenerator(generator.Generator):
         previous object with focus.
         """
         result = []
+        acss = self.voice(DEFAULT)
         if obj:
             priorObj = args.get('priorObj', None)
             try:
@@ -405,6 +571,8 @@ class SpeechGenerator(generator.Generator):
                        and ((newRow != oldRow) \
                             or (obj.parent != priorParent)):
                         result = self._generateRowHeader(obj, **args)
+        if result:
+            result.extend(acss)
         return result
 
     def _generateNewColumnHeader(self, obj, **args):
@@ -418,6 +586,7 @@ class SpeechGenerator(generator.Generator):
         previous object with focus.
         """
         result = []
+        acss = self.voice(DEFAULT)
         if obj and not args.get('readingRow', False):
             priorObj = args.get('priorObj', None)
             try:
@@ -451,6 +620,8 @@ class SpeechGenerator(generator.Generator):
                        and ((newCol != oldCol) \
                             or (obj.parent != priorParent)):
                         result = self._generateColumnHeader(obj, **args)
+        if result:
+            result.extend(acss)
         return result
 
     def _generateRealTableCell(self, obj, **args):
@@ -461,6 +632,7 @@ class SpeechGenerator(generator.Generator):
         cell itself.  The string, 'blank', is added for empty cells.
         """
         result = []
+        acss = self.voice(DEFAULT)
         oldRole = self._overrideRole('REAL_ROLE_TABLE_CELL', args)
         result.extend(self.generate(obj, **args))
         self._restoreRole(oldRole, args)
@@ -470,6 +642,9 @@ class SpeechGenerator(generator.Generator):
             # user has navigated to an empty line.
             #
             result.append(_("blank"))
+            if result:
+                result.extend(acss)
+
         return result
 
     def _generateUnselectedCell(self, obj, **args):
@@ -481,7 +656,7 @@ class SpeechGenerator(generator.Generator):
         settings.py.]]]
         """
         result = []
-
+        acss = self.voice(STATE)
         # If this is an icon within an layered pane or a table cell
         # within a table or a tree table and the item is focused but not
         # selected, let the user know. See bug #486908 for more details.
@@ -517,6 +692,7 @@ class SpeechGenerator(generator.Generator):
             # selected or not.
             #
             result.append(C_("tablecell", "not selected"))
+            result.extend(acss)
 
         return result
 
@@ -525,6 +701,7 @@ class SpeechGenerator(generator.Generator):
         specifications) reflecting the column number of a cell.
         """
         result = []
+        acss = self.voice(SYSTEM)
         col = -1
         if obj.parent.getRole() == pyatspi.ROLE_TABLE_CELL:
             obj = obj.parent
@@ -541,6 +718,8 @@ class SpeechGenerator(generator.Generator):
             # Translators: this is in references to a column in a
             # table.
             result.append(_("column %d") % (col + 1))
+        if result:
+            result.extend(acss)
         return result
 
     def _generateRow(self, obj, **args):
@@ -548,6 +727,7 @@ class SpeechGenerator(generator.Generator):
         specifications) reflecting the row number of a cell.
         """
         result = []
+        acss = self.voice(SYSTEM)
         row = -1
         if obj.parent.getRole() == pyatspi.ROLE_TABLE_CELL:
             obj = obj.parent
@@ -564,6 +744,8 @@ class SpeechGenerator(generator.Generator):
             # Translators: this is in references to a row in a table.
             #
             result.append(_("row %d") % (row + 1))
+        if result:
+            result.extend(acss)
         return result
 
     def _generateColumnAndRow(self, obj, **args):
@@ -573,6 +755,7 @@ class SpeechGenerator(generator.Generator):
         and the total number of rows.
         """
         result = []
+        acss = self.voice(SYSTEM)
         if obj.parent.getRole() == pyatspi.ROLE_TABLE_CELL:
             obj = obj.parent
         parent = obj.parent
@@ -594,6 +777,8 @@ class SpeechGenerator(generator.Generator):
             result.append(_("row %(index)d of %(total)d") \
                           % {"index" : (row + 1),
                              "total" : table.nRows})
+        if result:
+            result.extend(acss)
         return result
 
     def _generateEndOfTableIndicator(self, obj, **args):
@@ -602,6 +787,7 @@ class SpeechGenerator(generator.Generator):
         in the table.
         """
         result = []
+        acss = self.voice(SYSTEM)
         if settings.speechVerbosityLevel == settings.VERBOSITY_LEVEL_VERBOSE:
             if obj.getRole() == pyatspi.ROLE_TABLE_CELL:
                 cell = obj
@@ -621,6 +807,8 @@ class SpeechGenerator(generator.Generator):
                     # he/she is in the last cell of a table in a document.
                     #
                     result.append(_("End of table"))
+        if result:
+            result.extend(acss)
         return result
 
     #####################################################################
@@ -637,6 +825,7 @@ class SpeechGenerator(generator.Generator):
         to return an empty array if this is not a terminal.]]]
         """
         result = []
+        acss = self.voice(DEFAULT)
         title = None
         frame = self._script.utilities.ancestorWithRole(
             obj, [pyatspi.ROLE_FRAME], [])
@@ -645,6 +834,8 @@ class SpeechGenerator(generator.Generator):
         if not title:
             title = self._script.utilities.displayedLabel(obj)
         result.append(title)
+        if result:
+            result.extend(acss)
         return result
 
     #####################################################################
@@ -653,6 +844,18 @@ class SpeechGenerator(generator.Generator):
     #                                                                   #
     #####################################################################
 
+    def _generateCurrentLineText(self, obj, **args):
+        """Returns an array of strings for use by speech and braille
+        that represents the current line of text, if
+        this is a text object.  [[[WDW - consider returning an empty
+        array if this is not a text object.]]]
+        """
+        acss = self.voice(DEFAULT)
+        result = generator.Generator._generateCurrentLineText(self, obj, **args)
+        if result:
+            result.extend(acss)
+        return result
+
     def _getCharacterAttributes(self,
                                 obj,
                                 text,
@@ -792,15 +995,19 @@ class SpeechGenerator(generator.Generator):
         except NotImplementedError:
             return []
 
+        result = []
+        acss = self.voice(DEFAULT)
         [line, startOffset, endOffset, selected] = \
             self._getTextInformation(obj)
 
         # The empty string seems to be messing with using 'or' in
         # formatting strings.
         #
-        if line == '':
-            return []
-        return [line]
+        if line:
+            result.append(line)
+            result.extend(acss)
+
+        return result
 
     def _generateTextContentWithAttributes(self, obj, **args):
         """Returns an array of strings (and possibly voice and audio
@@ -809,11 +1016,13 @@ class SpeechGenerator(generator.Generator):
         mixed in.  This requires _generateTextInformation to have been
         called prior to this method.
         """
+
         try:
             text = obj.queryText()
         except NotImplementedError:
             return []
 
+        acss = self.voice(DEFAULT)
         [line, startOffset, endOffset, selected] = \
             self._getTextInformation(obj)
 
@@ -842,7 +1051,9 @@ class SpeechGenerator(generator.Generator):
                 newLine += " ; "
             newLine += attribs
 
-        return [newLine]
+        result = [newline]
+        result.extend(acss)
+        return result
 
     def _generateAnyTextSelection(self, obj, **args):
         """Returns an array of strings (and possibly voice and audio
@@ -851,6 +1062,7 @@ class SpeechGenerator(generator.Generator):
         moved to settings.py.]]]
         """
         result = []
+        acss = self.voice(SYSTEM)
 
         [line, startOffset, endOffset, selected] = \
             self._getTextInformation(obj)
@@ -861,6 +1073,7 @@ class SpeechGenerator(generator.Generator):
             #
             text = C_("text", "selected")
             result.append(text)
+            result.extend(acss)
         return result
 
     def _generateAllTextSelection(self, obj, **args):
@@ -870,6 +1083,7 @@ class SpeechGenerator(generator.Generator):
         moved to settings.py.]]]
         """
         result = []
+        acss = self.voice(SYSTEM)
         try:
             textObj = obj.queryText()
         except:
@@ -884,6 +1098,7 @@ class SpeechGenerator(generator.Generator):
                     # a document, Orca lets them know this.
                     #
                     result = [C_("text", "selected")]
+                    result.extend(acss)
         return result
 
     def generateTextIndentation(self, obj, **args):
@@ -897,6 +1112,7 @@ class SpeechGenerator(generator.Generator):
         - obj: the text object.
         - line: the string to check for spaces and tabs.
         """
+        acss = self.voice(SYSTEM)
         if not settings.enableSpeechIndentation:
             return []
         line =  args.get('alreadyFocused', "")
@@ -941,9 +1157,10 @@ class SpeechGenerator(generator.Generator):
                 break
             spaceCount  = tabCount = 0
 
-        if len(utterance):
-            return [utterance]
-        return []
+        result = [utterance]
+        if result and result[0]:
+            result.extend(acss)
+        return result
 
     #####################################################################
     #                                                                   #
@@ -961,10 +1178,12 @@ class SpeechGenerator(generator.Generator):
         focus.
         """
         result = []
+        acss = self.voice(SYSTEM)
         oldLevel = self._script.utilities.nodeLevel(args.get('priorObj', None))
         newLevel = self._script.utilities.nodeLevel(obj)
         if (oldLevel != newLevel) and (newLevel >= 0):
             result.extend(self._generateNodeLevel(obj, **args))
+            result.extend(acss)
         return result
 
     #####################################################################
@@ -980,6 +1199,7 @@ class SpeechGenerator(generator.Generator):
         should consider returning an empty array if there is no value.
         """
         result = []
+        acss = self.voice(SYSTEM)
         try:
             value = obj.queryValue()
         except NotImplementedError:
@@ -991,8 +1211,12 @@ class SpeechGenerator(generator.Generator):
                 * 100.0
             # Translators: this is the percentage value of a progress bar.
             #
-            percentage = _("%d percent") % percentValue
+            percentage = ngettext("%d percent",
+                                  "%d percent",
+                                  percentValue) % percentValue
             result.append(percentage)
+        if result:
+            result.extend(acss)
         return result
 
     #####################################################################
@@ -1015,6 +1239,7 @@ class SpeechGenerator(generator.Generator):
         # AT-SPI method calls.]]]
         #
         result = []
+        acss = self.voice(DEFAULT)
         priorObj = args.get('priorObj', None)
         if obj and obj.getRole() == pyatspi.ROLE_RADIO_BUTTON:
             radioGroupLabel = None
@@ -1036,6 +1261,7 @@ class SpeechGenerator(generator.Generator):
             if (not inSameGroup) and radioGroupLabel:
                 result.append(self._script.utilities.\
                                   displayedText(radioGroupLabel))
+                result.extend(acss)
         return result
 
     def _generateNumberOfChildren(self, obj, **args):
@@ -1046,6 +1272,7 @@ class SpeechGenerator(generator.Generator):
         be moved to settings.py.]]]
         """
         result = []
+        acss = self.voice(SYSTEM)
         childNodes = self._script.utilities.childNodes(obj)
         children = len(childNodes)
         if children:
@@ -1054,6 +1281,7 @@ class SpeechGenerator(generator.Generator):
             #
             itemString = ngettext("%d item", "%d items", children) % children
             result.append(itemString)
+            result.extend(acss)
         return result
 
     def _generateNoShowingChildren(self, obj, **args):
@@ -1065,6 +1293,7 @@ class SpeechGenerator(generator.Generator):
         settings.py.]]]
         """
         result = []
+        acss = self.voice(SYSTEM)
         hasItems = False
         for child in obj:
             state = child.getState()
@@ -1076,6 +1305,7 @@ class SpeechGenerator(generator.Generator):
             # or table.
             #
             result.append(_("0 items"))
+            result.extend(acss)
         return result
 
     def _generateNoChildren(self, obj, **args ):
@@ -1087,11 +1317,13 @@ class SpeechGenerator(generator.Generator):
         settings.py.]]]
         """
         result = []
+        acss = self.voice(SYSTEM)
         if not obj.childCount:
             # Translators: this is the number of items in a layered pane
             # or table.
             #
             result.append(_("0 items"))
+            result.extend(acss)
         return result
 
     def _generateSelectedItemCount(self, obj, **args):
@@ -1101,6 +1333,7 @@ class SpeechGenerator(generator.Generator):
         panel or a layered pane.
         """
         result = []
+        acss = self.voice(SYSTEM)
         # TODO - JD: Is there a better way to do this other than
         # hard-coding it in?
         #
@@ -1127,13 +1360,13 @@ class SpeechGenerator(generator.Generator):
                       % {"index" : totalSelectedItems,
                          "total" : childCount}
         result.append(countString)
-
+        result.extend(acss)
         result.append(self._script.formatting.getString(
                           mode='speech',
                           stringType='iconindex') \
                       % {"index" : currentItem,
                          "total" : childCount})
-
+        result.extend(acss)
         return result
 
     def _generateSelectedItems(self, obj, **args):
@@ -1160,6 +1393,7 @@ class SpeechGenerator(generator.Generator):
         settings.py.]]]
         """
         result = []
+        acss = self.voice(SYSTEM)
         # If this application has more than one unfocused alert or
         # dialog window, then speak '<m> unfocused dialogs'
         # to let the user know.
@@ -1173,6 +1407,7 @@ class SpeechGenerator(generator.Generator):
             result.append(ngettext("%d unfocused dialog",
                             "%d unfocused dialogs",
                             alertAndDialogCount) % alertAndDialogCount)
+            result.extend(acss)
         return result
 
     def _generateAncestors(self, obj, **args):
@@ -1187,6 +1422,7 @@ class SpeechGenerator(generator.Generator):
         previous object with focus.
         """
         result = []
+        acss = self.voice(DEFAULT)
         priorObj = args.get('priorObj', None)
         requireText = args.get('requireText', True)
         commonAncestor = self._script.utilities.commonAncestor(priorObj, obj)
@@ -1207,16 +1443,21 @@ class SpeechGenerator(generator.Generator):
                                 and 'Text' in pyatspi.listInterfaces(parent))):
                         text = self._script.utilities.displayedText(parent)
                     if text and len(text.strip()):
+                        roleInfo = self._generateRoleName(parent)
+                        if roleInfo:
+                            roleInfo.reverse()
                         # Push announcement of cell to the end
                         #
                         if parent.getRole() not in [pyatspi.ROLE_TABLE_CELL,
                                                     pyatspi.ROLE_FILLER]:
-                            result.extend(self._generateRoleName(parent))
+                            result.extend(roleInfo)
+                        result.extend(acss)
                         result.append(text)
                         if parent.getRole() == pyatspi.ROLE_TABLE_CELL:
-                            result.extend(self._generateRoleName(parent))
+                            result.extend(roleInfo)
                 parent = parent.parent
-        return result.reverse() or result
+        result.reverse()
+        return result
 
     def _generateNewAncestors(self, obj, **args):
         """Returns an array of strings (and possibly voice and audio
@@ -1269,6 +1510,7 @@ class SpeechGenerator(generator.Generator):
         object in a group.
         """
         result = []
+        acss = self.voice(SYSTEM)
         position = -1
         total = -1
 
@@ -1292,7 +1534,7 @@ class SpeechGenerator(generator.Generator):
                               stringType='groupindex') \
                           % {"index" : position,
                              "total" : total})
-
+            result.extend(acss)
         return result
 
     def _generatePositionInList(self, obj, **args):
@@ -1301,6 +1543,7 @@ class SpeechGenerator(generator.Generator):
         object in a list.
         """
         result = []
+        acss = self.voice(SYSTEM)
         position = -1
         index = 0
         total = 0
@@ -1374,7 +1617,7 @@ class SpeechGenerator(generator.Generator):
                               stringType='groupindex') \
                           % {"index" : position,
                              "total" : total})
-
+            result.extend(acss)
         return result
 
     def _generateDefaultButton(self, obj, **args):
@@ -1431,6 +1674,7 @@ class SpeechGenerator(generator.Generator):
         any unfocused dialog boxes.
         """
         result = []
+        acss = self.voice(DEFAULT)
         frame, dialog = self._script.utilities.frameAndDialog(obj)
         if frame:
             result.append(self._generateLabelAndName(frame))
@@ -1442,9 +1686,12 @@ class SpeechGenerator(generator.Generator):
             # Translators: this tells the user how many unfocused
             # alert and dialog windows that this application has.
             #
-            result.append(ngettext("%d unfocused dialog",
-                                   "%d unfocused dialogs",
-                                   alertAndDialogCount) % alertAndDialogCount)
+            dialogs = []
+            dialogs.append(ngettext("%d unfocused dialog",
+                                    "%d unfocused dialogs",
+                                    alertAndDialogCount) % alertAndDialogCount)
+            dialogs.extend(acss)
+            result.append(dialogs)
         return result
 
     #####################################################################
@@ -1459,10 +1706,13 @@ class SpeechGenerator(generator.Generator):
         or an empty array if no accelerator can be found.
         """
         result = []
+        acss = self.voice(SYSTEM)
         [mnemonic, shortcut, accelerator] = \
             self._script.utilities.mnemonicShortcutAccelerator(obj)
         if accelerator:
             result.append(accelerator)
+            result.extend(acss)
+
         return result
 
     def _generateMnemonic(self, obj, **args):
@@ -1471,6 +1721,7 @@ class SpeechGenerator(generator.Generator):
         an empty array if no mnemonic can be found.
         """
         result = []
+        acss = self.voice(SYSTEM)
         if settings.enableMnemonicSpeaking or args.get('forceMnemonic', False):
             [mnemonic, shortcut, accelerator] = \
                 self._script.utilities.mnemonicShortcutAccelerator(obj)
@@ -1480,6 +1731,8 @@ class SpeechGenerator(generator.Generator):
                 mnemonic = shortcut
             if mnemonic:
                 result = [mnemonic]
+                result.extend(acss)
+
         return result
 
     #####################################################################
@@ -1497,6 +1750,7 @@ class SpeechGenerator(generator.Generator):
         'forceTutorial' attribute of the args dictionary to True.
         """
         result = []
+        acss = self.voice(SYSTEM)
         alreadyFocused = args.get('alreadyFocused', False)
         forceTutorial = args.get('forceTutorial', False)
         result.extend(self._script.tutorialGenerator.getTutorial(
@@ -1511,6 +1765,8 @@ class SpeechGenerator(generator.Generator):
                         frame,
                         alreadyFocused,
                         forceTutorial))
+        if result and result[0]:
+            result.extend(acss)
         return result
 
     #####################################################################
@@ -1525,13 +1781,12 @@ class SpeechGenerator(generator.Generator):
     def _generateLineBreak(self, obj, **args):
         return LINE_BREAK
 
-    def voice(self, key=None):
+    def voice(self, key=None, **args):
         """Returns an array containing a voice.  The key is a value
         to be used to look up the voice in the settings.py:voices
-        dictionary.
+        dictionary. Other arguments can be passed in for future
+        decision making.
         """
-        try:
-            voice = settings.voices[key]
-        except:
-            voice = settings.voices[settings.DEFAULT_VOICE]
-        return [voice]
+
+        voicename = voiceType.get(key) or voiceType.get(DEFAULT)
+        return [settings.voices.get(voicename)]
